; Public domain tool to display Athlon64 / Opteron CPU temperature
; Written by Eric Auer
; Based on an algorithm from the Linux kernel (source license: GPL)
; See for example: linux-2.6.19/drivers/hwmon/k8temp.c
	org 100h
start:	mov ax,0b101h	; PCI BIOS check
	int 1ah
	jc nopci
	cmp ah,0
	jnz nopci
	cmp dx,4350h	; edx=20494350, actually
	jz yespci
nopci:	mov dx,oldmsg
	mov ah,9
	int 21h
	mov al,255
	jmp done

yespci:	mov al,'0'
	add al,bh	; major
	mov [pciver],al
	mov al,bl	; minor
	shr al,4
	add al,'0'
	mov [pciver+2],al
	mov al,bl
	and al,15
	add al,'0'
	mov [pciver+3],al

	mov ax,0b102h	; find device
	xor si,si	; find first instance
	mov dx,1022h	; AMD
	mov cx,1103h	; Athlon64 / Opteron misc control
	int 1ah		; returns addr in bh:bl5.bl3
	jc nok8
	cmp ah,0
	jnc yesk8
nok8:	mov dx,nok8msg
	mov ah,9
	int 21h
	mov al,254
	jmp done

yesk8:	mov dx,tempmsg
	mov ah,9
	int 21h
	mov al,0	; core 0 sensor 0
	call gettmp
	mov al,40h	; core 0 sensor 1
	call gettmp
	mov al,04h	; core 1 sensor 0
	call gettmp
	mov al,44h	; core 1 sensor 1
	call gettmp
	mov dx,msg2
	int 21h

	mov bx,[values]
	cmp bx,0
	jz novals
	xor dx,dx
	mov ax,[average]
	div bx
	sub ax,49	; temp relative to -49 degrees
	js negval
	jmp short done2
novals: mov al,252	; no temp
	jmp short done2
negval:	mov al,0	; negative temp
done2:	jmp done


gettmp:	push ax		; al can be 0, 4, 40h, 44h to select a sensor
	push bx
	mov ax,0b108h	; read byte
	mov di,0e4h	; temp sensor select
	int 1ah
	pop bx
	jc pcie2
	cmp ah,0
	jnz pcie2
	and cl,0bbh	; core 0, cpu 0
	pop ax
	or cl,al
	push bx
	mov ax,0b10bh	; write byte
	mov di,0e4h	; temp sensor select
	int 1ah
	pop bx
pcie2:	jc pcierr
	cmp ah,0
	jnz pcierr
	push bx
	mov ax,0b10ah	; read dword
	mov di,0e4h	; temp sensor
	int 1ah
	pop bx
	jc pcierr
	cmp ah,0
	jnz pcierr

	shr ecx,16	; third byte
	or cl,cl	; any temp?
	jz notemp
	inc word [values]
	add word [average],cx
	sub cl,49	; 0 means -49 degrees etc
	jns postmp
	mov dl,'-'
	mov ah,2	; show char
	int 21h
	neg cl
postmp:	cmp cl,100
	jb oktmp
	sub cl,100
	mov dl,'1'	; max 199 degrees supported
	mov ah,2	; show char
	int 21h
oktmp:	mov al,cl
	aam		; ah=high, al=low digit (assume max 99 degrees)
	mov cx,ax
	mov al,'0'
	add al,ch
	mov dl,al
	mov ah,2	; show char
	int 21h
	mov al,'0'
	add al,cl
	mov dl,al
	mov ah,2	; show char
	int 21h
	mov dl,' '
	mov ah,2	; show char
	int 21h
	ret

notemp:	mov dx,namsg	; write "n/a "
	mov ah,9
	int 21h
	ret

pcierr:	mov dx,errmsg
	mov ah,9
	int 21h
	mov al,253
;	jmp done

done:	push ax
	mov dx,donemsg
	mov ah,9
	int 21h
	pop ax
	mov ah,4ch	; exit with errorlevel AL
	int 21h

values	dw 0	; count of sensors
average	dw 0	; average temperature, relative to -49 degrees

oldmsg	db "No PCI BIOS",13,10,"$"
nok8msg	db "Only Athlon64 / Opteron supported",13,10,"$"
donemsg	db "[AMDTEMP public domain tool by Eric Auer 2007: shows AMD K8 CPU temperatures]",13,10,"$"
errmsg	db "PCI error",13,10,"$"
namsg	db "n/a $"
tempmsg	db "AMD CPU temperatures: $"
msg2	db 0f8h,"C [PCI BIOS "
pciver	db "2.10]",13,10,"$"


